# SPDX-License-Identifier: GPL-3.0-or-later
"""Tests for item instantiation."""
from graphs.item import DataItem, FillItem, ItemFactory, TextItem

import pytest


def test_new_from_dict_data_item():
    """Test if new_from_dict returns a DataItem with the correct name."""
    d = {
        "type": "DataItem",
        "name": "My Dataset",
        "data": ([0.0, 1.0, 2.0], [5.0, 6.0, 7.0], [1, 3, 2], [3, 1, 2]),
        "color": "#1A5FB4",
    }
    item = ItemFactory.new_from_dict(d)
    assert isinstance(item, DataItem)
    assert item.get_name() == "My Dataset"
    assert item.get_xdata().tolist() == [0.0, 1.0, 2.0]
    assert item.get_ydata().tolist() == [5.0, 6.0, 7.0]
    assert item.get_xerr().tolist() == [1, 3, 2]
    assert item.get_yerr().tolist() == [3, 1, 2]


def test_new_from_dict_text_item():
    """Test if new_from_dict returns a TextItem with the correct properties."""
    d = {
        "type": "TextItem",
        "name": "A Label",
        "text": "Hello",
        "xanchor": 0.5,
        "yanchor": 0.25,
        "color": "#000000",
    }
    item = ItemFactory.new_from_dict(d)
    assert isinstance(item, TextItem)
    assert item.props.text == "Hello"
    assert item.props.xanchor == pytest.approx(0.5)
    assert item.props.yanchor == pytest.approx(0.25)


def test_new_from_dict_fill_item():
    """Test if new_from_dict returns a FillItem from a dict."""
    d = {
        "type": "FillItem",
        "name": "Fill",
        "data": (None, None, None),
        "color": "#62A0EA",
        "alpha": 0.25,
    }
    item = ItemFactory.new_from_dict(d)
    assert isinstance(item, FillItem)


def test_new_from_dict_unknown_type_raises():
    """Test if new_from_dict raises ValueError for an unknown item type."""
    d = {"type": "BogusItem", "name": "X"}
    with pytest.raises(ValueError):
        ItemFactory.new_from_dict(d)


def test_data_item_default_data():
    """Test if DataItem defaults data to empty lists when not provided."""
    item = DataItem(name="Empty")
    xdata, ydata = item.get_xydata()
    assert xdata.tolist() == []
    assert ydata.tolist() == []


def test_data_item_default_err():
    """Test if DataItem defaults err None when not provided."""
    item = DataItem(name="NoErr")
    assert item.get_xerr() is None
    assert item.get_yerr() is None



# SPDX-License-Identifier: GPL-3.0-or-later
"""Tests for operations."""
from graphs.operations import DataHelper, DataOperations

import numpy

import pytest

XDATA = numpy.array([0, 1, 4, 5, 7, 8, 12, 3])  # no duplicate x values
YDATA = numpy.array([5, 2, 7, 1, 31, 5, 123, 156])


def is_sorted(lst):
    """Check if a list is sorted in ascending order."""
    return all(lst[i] <= lst[i + 1] for i in range(len(lst) - 1))


def test_sort_data_x_is_sorted():
    """Test if sort_data sorts x data in ascending order."""
    sorted_x, sorted_y = DataHelper.sort_data(XDATA, YDATA)
    assert is_sorted(sorted_x)


def test_sort_data_xy_remain_paired():
    """Test if sort_data keeps x and y values correctly paired."""
    sorted_x, sorted_y = DataHelper.sort_data(XDATA, YDATA)
    original = dict(zip(XDATA, YDATA))
    for x, y in zip(sorted_x, sorted_y):
        assert original[x] == y


def test_normalize():
    """Test if normalize scales ydata to a maximum value of 1."""
    xdata, ydata, _sort, _discard = DataOperations.normalize(
        None, XDATA, YDATA,
    )
    assert max(ydata) == 1


def test_translate_x():
    """Test if translate_x shifts all x values by the given offset."""
    xdata, ydata, _sort, _discard = DataOperations.translate_x(
        None, XDATA, YDATA, 15,
    )
    assert all(x_new == x_old + 15 for x_old, x_new in zip(XDATA, xdata))


def test_translate_y():
    """Test if translate_y shifts all y values by the given offset."""
    xdata, ydata, _sort, _discard = DataOperations.translate_y(
        None, XDATA, YDATA, 15,
    )
    assert all(y_new == y_old + 15 for y_old, y_new in zip(YDATA, ydata))


def test_multiply_x():
    """Test if multiply_x scales all x values by the given multiplier."""
    xdata, ydata, _sort, _discard = DataOperations.multiply_x(
        None, XDATA, YDATA, 15,
    )
    assert all(x_new == x_old * 15 for x_old, x_new in zip(XDATA, xdata))


def test_multiply_y():
    """Test if multiply_y scales all y values by the given multiplier."""
    xdata, ydata, _sort, _discard = DataOperations.multiply_y(
        None, XDATA, YDATA, 15,
    )
    assert all(y_new == y_old * 15 for y_old, y_new in zip(YDATA, ydata))


def test_center_at_maximum():
    """Test if center places the x of the maximum y value at zero."""
    xdata, ydata, _sort, _discard = DataOperations.center(
        None, XDATA, YDATA, 0,
    )
    y_max_index = numpy.argmax(YDATA)
    assert xdata[y_max_index] == 0


def test_center_at_middle():
    """Test if center places the midpoint of the x range at zero."""
    xdata, ydata, _sort, _discard = DataOperations.center(
        None, XDATA, YDATA, 1,
    )
    middle_value = (min(XDATA) + max(XDATA)) / 2
    assert all(
        x_new == x_old - middle_value for x_old, x_new in zip(XDATA, xdata)
    )


def test_derivative():
    """Test if derivative returns the correct gradient at each point."""
    xdata = [1, 2, 3, 4, 5]
    ydata = [5, 10, 15, 10, 5]
    _, y_new, _sort, _discard = DataOperations.derivative(None, xdata, ydata)
    assert len(y_new) == len(xdata)
    assert y_new == pytest.approx([5, 5, 0, -5, -5], rel=1e-6)


def test_integral():
    """Test if integral returns the correct cumulative area at each point."""
    xdata = [1, 2, 3, 4, 5]
    ydata = [5, 10, 15, 10, 5]
    x_new, y_new, _sort, _discard = DataOperations.integral(
        None, xdata, ydata,
    )
    assert len(y_new) == len(xdata)
    assert y_new == pytest.approx([0, 7.5, 20, 32.5, 40], rel=1e-6)


def test_cut_returns_none():
    """Test if cut returns no x and y lists."""
    xdata, ydata, _sort, _discard = DataOperations.cut(None, XDATA, YDATA)
    assert xdata is None
    assert ydata is None


def test_fft_output_length():
    """Test if fft returns arrays of the same length as the input."""
    xdata = list(range(8))
    ydata = [0, 1, 0, -1, 0, 1, 0, -1]
    x_new, y_new, _sort, _discard = DataOperations.fft(None, xdata, ydata)
    assert len(x_new) == len(xdata)
    assert len(y_new) == len(ydata)


def test_inverse_fft_output_length():
    """Test if inverse_fft returns arrays of the same length as the input."""
    xdata = list(range(8))
    ydata = [0, 1, 0, -1, 0, 1, 0, -1]
    x_new, y_new, _sort, _discard = DataOperations.inverse_fft(
        None, xdata, ydata,
    )
    assert len(x_new) == len(xdata)
    assert len(y_new) == len(ydata)



#!/usr/bin/python
# SPDX-License-Identifier: GPL-3.0-or-later
"""
Generate Graphs GResource.

Used at build time by meson, but is build-system-independent.
"""
import argparse
import importlib.util
import logging
import shutil
import sys
from pathlib import Path
from xml.etree import ElementTree

from PIL import Image

from gi.repository import Gio

from matplotlib import font_manager

import numpy

parser = argparse.ArgumentParser(description="Generate Graphs gresource.")
parser.add_argument(
    "out",
    help="the output file",
)
parser.add_argument(
    "dir",
    help="Path to build directory. Files provided will be copied there.",
)
parser.add_argument(
    "style_io",
    help="Path to `style_io.py`. Used to generate style previews.",
)
parser.add_argument(
    "--ui",
    required=True,
    nargs="+",
    dest="ui",
    help="List of UI files.",
)
parser.add_argument(
    "--styles",
    required=True,
    nargs="+",
    dest="styles",
    help="List of style files.",
)
parser.add_argument(
    "--other",
    required=True,
    nargs="+",
    dest="other",
    help="List of other files to include.",
)
parser.add_argument(
    "--icons",
    required=True,
    nargs="+",
    dest="icons",
    help="List of icon files.",
)
args = parser.parse_args()

# Check fonts
font_list = font_manager.findSystemFonts(fontpaths=None, fontext="ttf")
for font in font_list:
    try:
        font_manager.fontManager.addfont(font)
    except RuntimeError:
        logging.warning("Could not load %s", font)
# Disable matplotlib logging
logging.getLogger("matplotlib.font_manager").setLevel(logging.ERROR)

# dynamically import style_io
spec = importlib.util.spec_from_file_location("style_io", args.style_io)
style_io = importlib.util.module_from_spec(spec)
sys.modules["style_io"] = style_io
spec.loader.exec_module(style_io)

# GResource tree creation
gresources = ElementTree.Element("gresources")
main_prefix = "/se/sjoerd/Graphs/"
main_gresource = ElementTree.SubElement(
    gresources,
    "gresource",
    attrib={"prefix": main_prefix},
)
current_dir = Path(args.dir)

# Begin Other Section
for file in args.other:
    path = Path(shutil.copy(file, current_dir))
    element = ElementTree.SubElement(
        main_gresource,
        "file",
        attrib={
            "compressed": "True",
        },
    )
    element.text = str(path.relative_to(current_dir))
# End Other Section

# Begin style section
styles = []
style_paths = {}
style_prefix = main_prefix + "styles/"
styles_gresource = ElementTree.SubElement(
    gresources,
    "gresource",
    attrib={"prefix": style_prefix},
)
style_list = Path(current_dir, "styles.txt")
for style_path in args.styles:
    style_file = Path(shutil.copy(style_path, current_dir))
    style_element = ElementTree.SubElement(
        styles_gresource,
        "file",
        attrib={
            "compressed": "True",
        },
    )
    style_element.text = str(style_file.relative_to(current_dir))
    g_file = Gio.File.new_for_path(str(style_file))
    params = style_io.parse(g_file)
    stylename = params[1]["name"]
    out_path = Path(current_dir, style_file.name.replace(".mplstyle", ".png"))
    style_paths[stylename] = out_path
    with open(out_path, "wb") as out_file:
        style_io.create_preview(out_file, params, "png", 31)
    preview_element = ElementTree.SubElement(
        main_gresource,
        "file",
        attrib={
            "compressed": "True",
        },
    )
    preview_element.text = str(out_path.relative_to(current_dir))
    styles.append([
        stylename,
        style_prefix + style_file.name,
        main_prefix + out_path.name,
    ])
styles.sort(key=lambda x: x[0].casefold())
with open(style_list, "wt") as style_list_file:
    style_list_file.writelines(";".join(x) + "\n" for x in styles)
style_list_element = ElementTree.SubElement(
    main_gresource,
    "file",
    attrib={
        "compressed": "True",
    },
)
style_list_element.text = str(style_list.relative_to(current_dir))


def _to_array(file_path):
    with open(file_path, "rb") as file:
        return numpy.array(Image.open(file).convert("RGB"))


# Generate stitched system previews for Adwaita and Yaru
for sys_style in ("Adwaita", "Yaru"):
    light_array = _to_array(style_paths[sys_style])
    dark_array = _to_array(style_paths[sys_style + " Dark"])
    height, width = light_array.shape[0:2]
    stitched_array = numpy.concatenate(
        (light_array[:, :width // 2], dark_array[:, width // 2:]),
        axis=1,
    )
    stitched_image = Image.fromarray(stitched_array)
    out_path = Path(current_dir, "system-style-" + sys_style.lower() + ".png")
    with open(out_path, "wb") as file:
        stitched_image.save(file, "PNG")
    preview_element = ElementTree.SubElement(
        main_gresource,
        "file",
        attrib={
            "compressed": "True",
        },
    )
    preview_element.text = str(out_path.relative_to(current_dir))
# End style section

# Begin ui section
ui_gresource = ElementTree.SubElement(
    gresources,
    "gresource",
    attrib={"prefix": main_prefix},
)
for ui_file in args.ui:
    path = Path(current_dir, ui_file)
    ui_file_element = ElementTree.SubElement(
        ui_gresource,
        "file",
        attrib={
            "preprocess": "xml-stripblanks",
        },
    )
    ui_file_element.text = str(path.relative_to(current_dir))
# End ui section

# Begin icon section
icon_gresource = ElementTree.SubElement(
    gresources,
    "gresource",
    attrib={"prefix": main_prefix + "icons/scalable/actions/"},
)
for icon_file in args.icons:
    path = Path(shutil.copy(icon_file, current_dir))
    icon_file_element = ElementTree.SubElement(
        icon_gresource,
        "file",
        attrib={
            "preprocess": "xml-stripblanks",
        },
    )
    icon_file_element.text = str(path.relative_to(current_dir))
# End icon section

# Write
tree = ElementTree.ElementTree(gresources)
tree.write(args.out)



# SPDX-License-Identifier: GPL-3.0-or-later
"""Tests for project migration."""
import copy
import json
import os

from gi.repository import Gio, Graphs

from graphs.project import ProjectMigrator, ProjectValidator, read_project_file

import pytest

PROJECTFILE_V1 = os.path.join(os.path.dirname(__file__), "project_v1.graphs")


@pytest.fixture
def v1_project_dict():
    """Load the v1 project dict from the test fixture."""
    with open(PROJECTFILE_V1) as f:
        return json.load(f)


@pytest.fixture
def migrated_project_dict(v1_project_dict):
    """Return the migrated dict from the v1 fixture."""
    return ProjectMigrator(
        copy.deepcopy(v1_project_dict),
        Graphs.ProjectParseFlags.NONE,
    ).migrate()


@pytest.fixture
def validated_project(migrated_project_dict):
    """Return a ProjectValidator after validating the migrated dict."""
    validator = ProjectValidator(
        migrated_project_dict,
        Graphs.ProjectParseFlags.NONE,
    )
    validator.validate()
    return validator


def test_read_old_project_file():
    """Test if read_project_file parses the v1 file without errors."""
    file = Gio.File.new_for_path(PROJECTFILE_V1)
    result = read_project_file(
        file, Graphs.ProjectParseFlags.NONE,
    )
    assert "data" in result
    assert "figure-settings" in result


def test_migration_completes_without_error(migrated_project_dict):
    """Test if migrate returns a dict with the expected top-level keys."""
    assert "data" in migrated_project_dict
    assert "figure-settings" in migrated_project_dict


def test_migration_preserves_item_count(
    v1_project_dict,
    migrated_project_dict,
):
    """Test if migrate keeps the same number of items as the original."""
    assert (
        len(migrated_project_dict["data"]) == len(v1_project_dict["data"])
    )


def test_migration_preserves_item_names(
    v1_project_dict,
    migrated_project_dict,
):
    """Test if migrate keeps all item names unchanged."""
    original_names = {i["name"] for i in v1_project_dict["data"]}
    migrated_names = {i["name"] for i in migrated_project_dict["data"]}
    assert migrated_names == original_names


def test_migration_preserves_item_data(
    v1_project_dict,
    migrated_project_dict,
):
    """Test if migrate preserves xdata and ydata for each item."""
    for v1_item, item in zip(
        v1_project_dict["data"],
        migrated_project_dict["data"],
    ):
        xdata, ydata, _xerr, _yerr = item["data"]
        assert list(xdata) == v1_item["xdata"]
        assert list(ydata) == v1_item["ydata"]


def test_migration_result_passes_validation(migrated_project_dict):
    """Test if the migrated dict passes ProjectValidator without errors."""
    ProjectValidator(
        migrated_project_dict,
        Graphs.ProjectParseFlags.NONE,
    ).validate()


def test_validator_constructs_figure_settings(validated_project):
    """Test if validate correctly constructs a FigureSettings object."""
    assert isinstance(
        validated_project.figure_settings, Graphs.FigureSettings,
    )
